@@ -17,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -264,8 +264,7 @@ soap_process_reply(soap_rpc_t *sr)
 
 	if (GNET_PROPERTY(soap_debug) > 2) {
 		g_debug("SOAP \"%s\" at \"%s\": processing reply (%zu byte%s) HTTP %d",
-			sr->action, sr->url, sr->reply_len,
-			plural(sr->reply_len), sr->http_code);
+			sr->action, sr->url, PLURAL(sr->reply_len), sr->http_code);
 	}
 
 	/*
