@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2004, Christian Biere
- * Copyright (c) 2013, Raphael Manfredi
+ * Copyright (c) 2013, 2015, 2020 Raphael Manfredi
  *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
@@ -18,7 +18,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -31,7 +31,7 @@
  * @author Christian Biere
  * @date 2004
  * @author Raphael Manfredi
- * @date 2013
+ * @date 2013, 2015, 2020
  */
 
 #include "common.h"
@@ -41,15 +41,19 @@
 #include "aq.h"
 #include "ascii.h"
 #include "atoms.h"
+#include "cq.h"
 #include "debug.h"
 #include "fd.h"
 #include "glib-missing.h"
 #include "hikset.h"
 #include "host_addr.h"
 #include "inputevt.h"
+#include "mutex.h"
 #include "once.h"
 #include "pslist.h"
+#include "shuffle.h"
 #include "signal.h"
+#include "stringify.h"
 #include "thread.h"
 #include "tm.h"
 #include "waiter.h"
@@ -149,10 +153,16 @@ typedef struct adns_cache_struct {
 	unsigned pos;
 	int timeout;
 	adns_cache_entry_t *entries[ADNS_CACHE_MAX_SIZE];
+	cperiodic_t *ev;
+	mutex_t lock;
 } adns_cache_t;
 
 static adns_cache_t *adns_cache = NULL;
 
+#define ADNS_CACHE_LOCK(ac)			mutex_lock(&(ac)->lock)
+#define ADNS_CACHE_UNLOCK(ac)		mutex_unlock(&(ac)->lock)
+#define ADNS_CACHE_IS_LOCKED(ac)	mutex_is_owned(&(ac)->lock)
+
 /* private variables */
 
 static unsigned adns_reply_event_id;
@@ -160,10 +170,25 @@ static aqueue_t *adns_req;
 static aqueue_t *adns_ans;
 static int adns_id = -1;
 
+static uint32 adns_debug;	/**< Debug level */
+
+#define adns_debugging(lvl)   G_UNLIKELY(adns_debug > (lvl))
+
+/**
+ * Set debug level.
+ */
+void
+set_adns_debug(uint32 level)
+{
+	adns_debug = level;
+}
+
 /**
  * Private functions.
  */
 
+static bool adns_cache_periodic(void *o);
+
 static adns_cache_t *
 adns_cache_init(void)
 {
@@ -178,6 +203,10 @@ adns_cache_init(void)
 	for (i = 0; i < N_ITEMS(cache->entries); i++) {
 		cache->entries[i] = NULL;
 	}
+	mutex_init(&cache->lock);
+	cache->ev = cq_periodic_main_add(
+		1000 * ADNS_CACHE_TIMEOUT,		/* in ms for debug */
+		adns_cache_periodic, cache);
 	return cache;
 }
 
@@ -186,9 +215,9 @@ adns_cache_init(void)
 #undef ADNS_CACHE_TIMEOUT
 
 static inline adns_cache_entry_t *
-adns_cache_get_entry(adns_cache_t *cache, unsigned i)
+adns_cache_get_entry(const adns_cache_t *cache, unsigned i)
 {
-	adns_cache_entry_t *entry;
+	const adns_cache_entry_t *entry;
 
 	g_assert(cache);
 	g_assert(i < N_ITEMS(cache->entries));
@@ -199,7 +228,18 @@ adns_cache_get_entry(adns_cache_t *cache, unsigned i)
 		g_assert(entry->hostname);
 		g_assert(entry->n > 0);
 	}
-	return entry;
+	return deconstify_pointer(entry);
+}
+
+static void
+adns_cache_free_item(adns_cache_entry_t *entry)
+{
+	g_assert(entry != NULL);
+	g_assert(entry->hostname != NULL);
+	g_assert(entry->n > 0);
+
+	atom_str_free_null(&entry->hostname);
+	wfree(entry, adns_cache_entry_size(entry->n));
 }
 
 static void
@@ -209,19 +249,117 @@ adns_cache_free_entry(adns_cache_t *cache, unsigned i)
 
 	g_assert(cache);
 	g_assert(i < N_ITEMS(cache->entries));
+	g_assert(ADNS_CACHE_IS_LOCKED(cache));
 
 	entry = cache->entries[i];
-	if (entry) {
+	if (entry != NULL) {
 		g_assert(i == entry->id);
-		g_assert(entry->hostname);
-		g_assert(entry->n > 0);
-
-		atom_str_free_null(&entry->hostname);
-		wfree(entry, adns_cache_entry_size(entry->n));
+		adns_cache_free_item(entry);
 		cache->entries[i] = NULL;
 	}
 }
 
+/**
+ * Drop entry from cache.
+ *
+ * The entry is freed and its cache slot cleared.
+ */
+static void
+adns_cache_drop_entry(
+	adns_cache_t *cache, adns_cache_entry_t *entry, const char *caller)
+{
+	g_assert(cache != NULL);
+	g_assert(entry != NULL);
+	g_assert(ADNS_CACHE_IS_LOCKED(cache));
+
+	if (adns_debugging(1)) {
+		s_debug("%s(): removing \"%s\" from cache (%zu entr%s) at slot #%u",
+			caller, entry->hostname, PLURAL_Y(entry->n), entry->id);
+	}
+
+	hikset_remove(cache->ht, entry->hostname);
+	adns_cache_free_entry(cache, entry->id);
+}
+
+/**
+ * Clear entry from cache.
+ *
+ * The slot is cleared and the entry de-indexed but it is not freed.
+ */
+static void
+adns_cache_clear_entry(adns_cache_t *cache, const adns_cache_entry_t *entry)
+{
+	g_assert(cache != NULL);
+	g_assert(entry != NULL);
+	g_assert(ADNS_CACHE_IS_LOCKED(cache));
+
+	hikset_remove(cache->ht, entry->hostname);
+	cache->entries[entry->id] = NULL;
+}
+
+/**
+ * Has cache entry expired?
+ */
+static inline bool
+adns_cache_expired_item(
+	const adns_cache_t *cache, const adns_cache_entry_t *entry, time_t now)
+{
+	return delta_time(now, entry->timestamp) >= cache->timeout;
+}
+
+/**
+ * Callout queue periodic event to prune expired cache entries.
+ */
+static bool
+adns_cache_periodic(void *o)
+{
+	adns_cache_t *cache = o;
+	time_t now = tm_time();
+	pslist_t *to_free = NULL;
+	size_t i, held = 0, expired = 0;
+
+	ADNS_CACHE_LOCK(cache);
+
+	for (i = 0; i < N_ITEMS(cache->entries); i++) {
+		const adns_cache_entry_t *entry = cache->entries[i];
+
+		if (NULL == entry)
+			continue;
+
+		g_assert(entry->id == i);
+
+		if (adns_cache_expired_item(cache, entry, now)) {
+			to_free = pslist_prepend_const(to_free, entry);
+			expired++;
+
+			if (adns_debugging(4)) {
+				s_debug(
+					"%s(): expiring \"%s\" from cache (%zu entr%s) at slot #%u",
+					G_STRFUNC, entry->hostname, PLURAL_Y(entry->n), entry->id);
+			}
+
+			/* Not adns_cache_drop_entry(), we'll free entry later */
+			adns_cache_clear_entry(cache, entry);
+		} else {
+			held++;
+		}
+	}
+
+	ADNS_CACHE_UNLOCK(cache);
+
+	/* Free items once the cache is no longer locked */
+
+	PSLIST_FOREACH_CALL(to_free, adns_cache_free_item);
+	pslist_free_null(&to_free);
+
+	if (adns_debugging(3)) {
+		s_debug("%s(): cache holds %zu entr%s (expired %zu this cycle)",
+			G_STRFUNC, PLURAL_Y(held), expired);
+	}
+
+	return TRUE;		/* Keep calling */
+}
+
 /**
  * Frees all memory allocated by the cache and returns NULL.
  */
@@ -238,10 +376,14 @@ adns_cache_free(adns_cache_t **cache_ptr)
 	g_assert(cache);
 	g_assert(cache->ht);
 
+	ADNS_CACHE_LOCK(cache);
+
 	for (i = 0; i < N_ITEMS(cache->entries); i++) {
 		adns_cache_free_entry(cache, i);
 	}
 	hikset_free_null(&cache->ht);
+	cq_periodic_remove(&cache->ev);
+	mutex_destroy(&cache->lock);
 	XFREE_NULL(cache);
 }
 
@@ -265,14 +407,20 @@ adns_cache_add(adns_cache_t *cache, time_t now,
 	g_assert(!hikset_contains(cache->ht, hostname));
 	g_assert(cache->pos < N_ITEMS(cache->entries));
 
+	if (adns_debugging(0)) {
+		s_debug("%s(): caching %zu entr%s for \"%s\" at slot #%u",
+			G_STRFUNC, PLURAL_Y(n), hostname, cache->pos);
+	}
+
+	ADNS_CACHE_LOCK(cache);
+
 	entry = adns_cache_get_entry(cache, cache->pos);
-	if (entry) {
+
+	if (entry != NULL) {
 		g_assert(entry->hostname);
 		g_assert(entry == hikset_lookup(cache->ht, entry->hostname));
 
-		hikset_remove(cache->ht, entry->hostname);
-		adns_cache_free_entry(cache, cache->pos);
-		entry = NULL;
+		adns_cache_drop_entry(cache, entry, G_STRFUNC);
 	}
 
 	entry = walloc(adns_cache_entry_size(n));
@@ -282,22 +430,37 @@ adns_cache_add(adns_cache_t *cache, time_t now,
 	entry->id = cache->pos;
 	for (i = 0; i < entry->n; i++) {
 		entry->addrs[i] = addrs[i];
+
+		if (adns_debugging(1)) {
+			s_debug("%s(): caching \"%s\" for \"%s\" (#%zu/%zu)",
+				G_STRFUNC, host_addr_to_string(addrs[i]), hostname, i + 1, n);
+		}
 	}
+
+	if (adns_debugging(3)) {
+		s_debug("%s(): caching \"%s\" at slot #%u/%zu",
+			G_STRFUNC, hostname, cache->pos, N_ITEMS(cache->entries));
+	}
+
 	hikset_insert_key(cache->ht, &entry->hostname);
 	cache->entries[cache->pos++] = entry;
 	cache->pos %= N_ITEMS(cache->entries);
+
+	ADNS_CACHE_UNLOCK(cache);
 }
 
 /**
- * Looks for ``hostname'' in ``cache'' wrt to cache->timeout. If
- * ``hostname'' is not found or the entry is expired, FALSE will be
+ * Looks for ``hostname'' in ``cache'' .
+ *
+ * If ``hostname'' is not found or the entry is expired, FALSE will be
  * returned. Expired entries will be removed! ``addr'' is allowed to
  * be NULL, otherwise the cached IP will be stored into the variable
  * ``addr'' points to.
  *
- * @param addrs An array of host_addr_t items. If not NULL, up to
- *              ``n'' items will be copied from the cache.
- * @param n The number of items "addrs" can hold.
+ * @param addrs		An array of host_addr_t items. If not NULL, up to
+ *              	``n'' items will be copied from the cache.
+ * @param n			The number of items "addrs" can hold.
+ *
  * @return The number of cached addresses for the given hostname.
  */
 static size_t
@@ -305,39 +468,63 @@ adns_cache_lookup(adns_cache_t *cache, time_t now,
 	const char *hostname, host_addr_t *addrs, size_t n)
 {
 	adns_cache_entry_t *entry;
+	size_t n_entry = 0;
 
 	g_assert(NULL != cache);
 	g_assert(NULL != hostname);
 	g_assert(0 == n || NULL != addrs);
 
+	ADNS_CACHE_LOCK(cache);
+
 	entry = hikset_lookup(cache->ht, hostname);
-	if (entry) {
-		if (delta_time(now, entry->timestamp) < cache->timeout) {
+
+	if (entry != NULL) {
+		if (!adns_cache_expired_item(cache, entry, now)) {
 			size_t i;
 
+			if (adns_debugging(0)) {
+				s_debug("%s(): \"%s\" is cached with %zu entr%s",
+					G_STRFUNC, entry->hostname, PLURAL_Y(entry->n));
+			}
+
 			for (i = 0; i < n; i++) {
 				if (i < entry->n) {
 					addrs[i] = entry->addrs[i];
-					if (common_dbg > 0)
-						g_debug("%s: \"%s\" cached (addr=%s)", G_STRFUNC,
-							entry->hostname, host_addr_to_string(addrs[i]));
+
+					if (adns_debugging(1)) {
+						s_debug("%s(): \"%s\" returning cached addr=%s (#%zu/%zu)",
+							G_STRFUNC, entry->hostname,
+							host_addr_to_string(addrs[i]),
+							i + 1, entry->n);
+					}
 				} else {
 					addrs[i] = zero_host_addr;
 				}
 			}
+			n_entry = MIN(entry->n, n);		/* Amount of addresses copied */
 		} else {
-			if (common_dbg > 0) {
-				g_debug("%s: removing \"%s\" from cache",
-					G_STRFUNC, entry->hostname);
-			}
-
-			hikset_remove(cache->ht, hostname);
-			adns_cache_free_entry(cache, entry->id);
-			entry = NULL;
+			adns_cache_drop_entry(cache, entry, G_STRFUNC);
 		}
 	}
 
-	return entry ? entry->n : 0;
+	ADNS_CACHE_UNLOCK(cache);
+
+	/*
+	 * If there are more than one address associated with the host, a
+	 * real DNS would rotate the answer so that we do not hit the same
+	 * IP address, ever.
+	 *
+	 * Here, we simply shuffle the results to ensure we do not hit the
+	 * same IP address in the unlikely event a name resolves to a
+	 * server pool and not to a load balancer or a single server.
+	 *
+	 * 		--RAM, 2020-04-06
+	 */
+
+	if (n_entry > 1)
+		SHUFFLE_ARRAY_N(addrs, n_entry);
+
+	return n_entry;
 }
 
 /**
@@ -358,8 +545,8 @@ adns_gethostbyname(const struct adns_request *req, struct adns_response *ans)
 		struct adns_reverse_reply *reply = &ans->reply.reverse;
 		const char *host;
 
-		if (common_dbg > 1) {
-			s_debug("%s: reverse-resolving \"%s\" ...",
+		if (adns_debugging(1)) {
+			s_debug("%s(): reverse-resolving \"%s\" ...",
 					G_STRFUNC, host_addr_to_string(query->addr));
 		}
 
@@ -372,8 +559,8 @@ adns_gethostbyname(const struct adns_request *req, struct adns_response *ans)
 		pslist_t *sl_addr, *sl;
 		size_t i = 0;
 
-		if (common_dbg > 1) {
-			s_debug("%s: resolving \"%s\" ...", G_STRFUNC, query->hostname);
+		if (adns_debugging(1)) {
+			s_debug("%s(): resolving \"%s\" ...", G_STRFUNC, query->hostname);
 		}
 		clamp_strcpy(ARYLEN(reply->hostname), query->hostname);
 
@@ -436,7 +623,8 @@ adns_helper(void *p)
 		aq_put(aq, ans);
 	}
 
-	s_debug("ADNS thread exiting");
+	if (adns_debugging(0))
+		s_debug("ADNS thread exiting");
 
 	aq_refcnt_dec(rq);
 	aq_refcnt_dec(aq);
@@ -489,10 +677,10 @@ adns_reply_ready(const struct adns_response *ans)
 	g_assert(ans != NULL);
 
 	if (ans->common.reverse) {
-		if (common_dbg > 1) {
+		if (adns_debugging(1)) {
 			const struct adns_reverse_reply *reply = &ans->reply.reverse;
 
-			g_debug("%s: resolved \"%s\" to \"%s\".",
+			s_debug("%s(): resolved \"%s\" to \"%s\".",
 				G_STRFUNC, host_addr_to_string(reply->addr), reply->hostname);
 		}
 	} else {
@@ -502,19 +690,22 @@ adns_reply_ready(const struct adns_response *ans)
 		num = count_addrs(reply->addrs, N_ITEMS(reply->addrs));
 		num = MAX(1, num); /* For negative caching */
 
-		if (common_dbg > 1) {
+		if (adns_debugging(1)) {
 			size_t i;
 
 			for (i = 0; i < num; i++) {
-				g_debug("%s: resolved \"%s\" to \"%s\".", G_STRFUNC,
+				s_debug("%s(): resolved \"%s\" to \"%s\".", G_STRFUNC,
 					reply->hostname, host_addr_to_string(reply->addrs[i]));
 			}
 		}
 
+		ADNS_CACHE_LOCK(adns_cache);
 
 		if (!adns_cache_lookup(adns_cache, now, reply->hostname, NULL, 0)) {
 			adns_cache_add(adns_cache, now, reply->hostname, reply->addrs, num);
 		}
+
+		ADNS_CACHE_UNLOCK(adns_cache);
 	}
 
 	g_assert(ans->common.user_callback);
@@ -615,7 +806,7 @@ adns_init(void)
 }
 
 /**
- * @return TRUE on success, FALSE on failure.
+ * @return TRUE on success, FALSE if the ADNS queue is gone.
  */
 static bool
 adns_send_request(const struct adns_request *req)
@@ -624,8 +815,8 @@ adns_send_request(const struct adns_request *req)
 
 	g_assert(req != NULL);
 
-	if (0 == adns_reply_event_id)
-		return FALSE;
+	if (NULL == adns_req)
+		return FALSE;			/* ADNS thread was shutdown */
 
 	r = WCOPY(req);
 	aq_put(adns_req, r);
@@ -702,8 +893,8 @@ adns_resolve(const char *hostname, enum net_type net,
 	if (adns_send_request(&req))
 		return TRUE; /* asynchronous */
 
-	if (common_dbg) {
-		g_warning("%s(): using synchronous resolution for \"%s\"",
+	if (adns_debugging(0)) {
+		s_warning("%s(): falling back to synchronous resolution for \"%s\"",
 			G_STRFUNC, query->hostname);
 	}
 	adns_fallback(&req);
@@ -742,7 +933,7 @@ adns_reverse_lookup(const host_addr_t addr,
 	if (adns_send_request(&req))
 		return TRUE; /* asynchronous */
 
-	g_warning("%s(): using synchronous resolution for \"%s\"",
+	s_warning("%s(): using synchronous resolution for \"%s\"",
 		G_STRFUNC, host_addr_to_string(query->addr));
 
 	adns_fallback(&req);
@@ -756,6 +947,9 @@ adns_reverse_lookup(const host_addr_t addr,
 void
 adns_close(void)
 {
+	if (adns_debugging(0))
+		s_debug("%s(): terminating ADNS, thread #%d", G_STRFUNC, adns_id);
+
 	aq_put(adns_req, NULL);		/* Signals: end of processing */
 	aq_destroy_null(&adns_req);
 	aq_destroy_null(&adns_ans);
@@ -771,8 +965,12 @@ adns_close(void)
 	 */
 
 	if (-1 != adns_id) {
-		if (-1 == thread_join(adns_id, NULL))
-			g_warning("%s(): cannot join with ADNS thread: %m", G_STRFUNC);
+		if (-1 == thread_join(adns_id, NULL)) {
+			s_warning("%s(): cannot join with ADNS thread: %m", G_STRFUNC);
+		} else {
+			if (adns_debugging(0))
+				s_debug("%s(): ADNS thread #%d terminated", G_STRFUNC, adns_id);
+		}
 		adns_id = -1;
 	}
 }
