@@ -18,7 +18,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -1423,7 +1423,7 @@ compact_time(time_delta_t t)
 const char *
 compact_time2(time_delta_t t)
 {
-	buf_t *b = buf_private(G_STRFUNC, SIZE_FIELD_MAX);
+	buf_t *b = buf_private(G_STRFUNC, COMPACT_TIME_MAX_LEN);
 	char *p = buf_data(b);
 	size_t n, sz = buf_size(b);
 
@@ -1571,4 +1571,40 @@ time_locale_to_string_buf(time_t t, char *dst, size_t size)
 	return len;
 }
 
+/**
+ * Convert boolean to string.
+ *
+ * When the value given is not simply TRUE or FALSE, show the actual value,
+ * which is of course TRUE.
+ *
+ * @param b		the boolean value
+ */
+const char *
+bool_to_string(bool v)
+{
+	if (0 == v)
+		return "FALSE";
+	else if (1 == v)
+		return "TRUE";
+	else {
+		/* Catch any boolean which is not simply 0 or 1 */
+		buf_t *b = buf_private(G_STRFUNC, INT_DEC_BUFLEN + sizeof("TRUE="));
+		char *p = buf_data(b);
+		size_t sz = buf_size(b);
+
+		/*
+		 * We carp to help find the source of the culprit, knowing that
+		 * if several booleans are logged in a single formatting call,
+		 * only the last value will actually be printed if several have
+		 * non TRUE or FALSE values (they all use the same private buffer
+		 * to hold the value).
+		 */
+
+		s_carp_once("%s(): actual boolean value is %d", G_STRFUNC, v);
+
+		str_bprintf(p, sz, "TRUE=%d", v);	/* Visual indication of weirdness */
+		return p;
+	}
+}
+
 /* vi: set ts=4 sw=4 cindent: */
